@@ -12,7 +12,7 @@ package Mojolicious::Plugin::BootstrapHelpers {
 
     use experimental 'postderef';
 
-    our $VERSION = 0.007;
+    our $VERSION = 0.008;
 
     sub bootstraps_bootstraps {
         my $c = shift;
@@ -39,6 +39,21 @@ package Mojolicious::Plugin::BootstrapHelpers {
         
         $attr = add_classes($attr, 'panel', { panel => 'panel-%s', panel_default => 'default'});
         
+        my $body = qq{
+                <div class="panel-body">
+                    } . contents($callback, $content) . qq{
+                </div>
+        };
+
+        return create_panel($title, $body, $attr);
+
+    }
+
+    sub create_panel {
+        my $title = shift;
+        my $body = shift;
+        my $attr = shift;
+        
         my $tag = qq{
             <div class="$attr->{'class'}">
             } . (defined $title ? qq{
@@ -46,31 +61,33 @@ package Mojolicious::Plugin::BootstrapHelpers {
                     <h3 class="panel-title">$title</h3>
                 </div>
             } : '') . qq{
-                <div class="panel-body">
-                    } . contents($callback, $content) . qq{
-                </div>
+                $body
             </div>
         };
 
         return out($tag);
-
     }
 
     sub bootstrap_table {
         my $c = shift;
         my $callback = ref $_[-1] eq 'CODE' ? pop : undef;
-        my $content = undef; #scalar @_ % 2 ? pop : '';
+        my $title = scalar @_ % 2 ? shift : undef;
         my $attr = parse_attributes(@_);
-       
+        
         $attr = add_classes($attr, 'table', { table => 'table-%s' });
 
-        my $tag = qq{
+        my $table = qq{
             <table class="$attr->{'class'}">
-            } . contents($callback, $content) . qq{
+            } . $callback->() . qq{
             </table>
         };
 
-        return out($tag);
+        if(defined $title) {
+            $attr->{'panel'} = add_classes($attr->{'panel'}, 'panel', { panel => 'panel-%s', panel_default => 'default'});
+        }
+
+
+        return defined $title ? create_panel($title, $table, $attr->{'panel'}) : out($table);
     }
 
     sub bootstrap_formgroup {
@@ -109,7 +126,7 @@ package Mojolicious::Plugin::BootstrapHelpers {
 
         # We have an url
         if(scalar @url) {
-            $attr->{'href'} = $c->url_for(@url);
+            $attr->{'href'} = $c->url_for(@url)->to_string;
             return out(Mojolicious::Plugin::TagHelpers::_tag('a', $attr->%*, $content));
         }
         else {
@@ -305,8 +322,6 @@ package Mojolicious::Plugin::BootstrapHelpers {
 
     sub out {
         my $tag = shift;
-        $tag =~ s{>[ \n\t\s]+<}{><}mg;
-        $tag =~ s{[ \s]+$}{}g;
         return Mojo::ByteStream->new($tag);
     }
 
@@ -315,24 +330,24 @@ package Mojolicious::Plugin::BootstrapHelpers {
         my $app = shift;
         my $args = shift;
 
-        my $px = setup_prefix($args->{'tag_prefix'});
-        my $spx = setup_prefix($args->{'shortcut_prefix'});
-        my $init_shortcuts = $args->{'init_shortcuts'} //= 1;
+        my $tp = setup_prefix($args->{'tag_prefix'});
+        my $ssp = setup_prefix($args->{'short_strappings_prefix'});
+        my $init_short_strappings = $args->{'init_short_strappings'} //= 1;
 
-        $app->helper($px.'bootstrap' => \&bootstraps_bootstraps);
-        $app->helper($px.'table' => \&bootstrap_table);
-        $app->helper($px.'panel' => \&bootstrap_panel);
-        $app->helper($px.'formgroup' => \&bootstrap_formgroup);
-        $app->helper($px.'button' => \&bootstrap_button);
-        $app->helper($px.'submit_button' => \&bootstrap_submit);
+        $app->helper($tp.'bootstrap' => \&bootstraps_bootstraps);
+        $app->helper($tp.'table' => \&bootstrap_table);
+        $app->helper($tp.'panel' => \&bootstrap_panel);
+        $app->helper($tp.'formgroup' => \&bootstrap_formgroup);
+        $app->helper($tp.'button' => \&bootstrap_button);
+        $app->helper($tp.'submit_button' => \&bootstrap_submit);
 
-        if($init_shortcuts) {
+        if($init_short_strappings) {
             my @sizes = qw/xsmall small medium large/;
             my @contexts = qw/default primary success info warning danger/;
             my @table = qw/striped bordered hover condensed responsive/;
 
             foreach my $helper (@sizes, @contexts, @table) {
-               $app->helper($spx.$helper, sub { ("__$helper" => 1) });
+               $app->helper($ssp.$helper, sub { ("__$helper" => 1) });
             }
         }
     }
@@ -439,16 +454,23 @@ It is also possible to automatically include jQuery (2.*)
     <script src="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/js/bootstrap.min.js"></script>
 
 
-=head2 Shortcuts
+=head2 Strappings
 
-There are several shortcuts for context and size classes, that automatically expands to the correct class depending on which tag it is applied to.
-They can be seen as a hash key and value merged into one.
+There are several shortcuts ("strappings") for applying context and size classes that automatically expands to the correct class depending 
+on which tag it is applied to. For instance, if you apply the C<info> strapping to a panel, it becomes C<panel-info>, but when applied to a button it becomes C<btn-info>.
 
-For instance, if you apply the C<info> shortcut to a panel, it becomes C<panel-info>, but when applied to a button it becomes C<btn-info>.
+You can use them in two different ways, but internally they are the same. These to lines are exactly identical:
 
-For sizes, you can only use the longform (C<xsmall>, C<small>, C<medium> and C<large>), they are shortened to the Bootstrap type classes.
+    %= button 'Push me', primary
 
-The following shortcuts are available:
+    %= button 'Push me', __primary => 1
+
+
+
+For sizes, you can only use the longform (C<xsmall>, C<small>, C<medium> and C<large>) no matter if you use the short strapping form or not.
+They are shortened to the Bootstrap type classes.
+
+The following strappings are available:
 
    xsmall    default     striped
    small     primary     bordered
@@ -457,26 +479,40 @@ The following shortcuts are available:
              warning     responsive
              danger
 
-See below for usage. B<Important:> You can't follow a shortcut with a fat comma (C<=E<gt>>). The fat comma auto-quotes the shortcut, and then the shortcut is not a shortcut anymore.
+Add two leading underscores if you don't want to use the short form.
 
-If there is no corresponding class for the element you add the shortcut to it is silently not applied.
+See below for usage. B<Important:> You can't follow a short form strapping with a fat comma (C<=E<gt>>). The fat comma auto-quotes the strapping, and then it breaks.
+
+If there is no corresponding class for the element you add the strapping to it is silently not applied.
 
 =begin html
 
-You can turn off shortcuts, see <a href="#init_shortcuts">init_shortcuts</a>.
+<p>The short form is recommended for readability, but it does setup several helpers in your templates. 
+You can turn off the short forms, see <a href="#init_short_strappings">init_short_strappings</a>.</p>
 
 =end html
 
+=head2 Syntax convention
+
+In the syntax sections below the following conventions are used:
+    
+    name      A specific string
+    $name     Any string
+    $name[]   An array reference  (ordering significant)
+    %name     A hash              (ordering not significant)
+    $name{}   A hash reference    (ordering not significant)
+
+
 
 =head2 Panels
 
 L<Bootstrap documentation|http://getbootstrap.com/components/#panels>
 
-=head4 Syntax
+=head3 Syntax
 
     %= panel
 
-    %= panel $title, @shortcuts, begin
+    %= panel $title, %strappings, begin
         $body
     %  end
 
@@ -484,9 +520,9 @@ B<C<$title>>
 
 Usually mandatory, but can be omitted if there are no other arguments to the C<panel>. Otherwise, if you don't want a title, set it C<undef>.
 
-B<C<@shortcuts>>
+B<C<%strappings>>
 
-Optional hash. Any shortcuts you want applied to the C<panel>.
+Optional hash. Any strapping you want applied to the C<panel>.
 
 B<C<$body>>
 
@@ -495,7 +531,7 @@ Optional (but panels are not much use without it). The html inside the C<panel>.
 
 =head3 Examples
 
-=head4 No body, no title
+B<No body, no title>
 
     %= panel
 
@@ -506,7 +542,7 @@ Optional (but panels are not much use without it). The html inside the C<panel>.
 
 The class is set to C<panel-default>, by default.
 
-=head4 Body, no title
+B<Body, no title>
 
     %= panel undef ,=> begin
         <p>A short text.</p>
@@ -520,7 +556,7 @@ The class is set to C<panel-default>, by default.
 
 If you want a panel without title, set the title to C<undef>. Note that you can't use a regular fat comma since that would turn undef into a string. A normal comma is of course also ok.
 
-=head4 Body and title
+B<Body and title>
 
     %= panel 'The header' => begin
         <p>A short text.</p>
@@ -535,7 +571,7 @@ If you want a panel without title, set the title to C<undef>. Note that you can'
         </div>
     </div>
 
-=head4 Body and title, with context
+B<Body and title, with context>
     
     %= panel 'Panel 5', success, begin
         <p>A short text.</p>
@@ -550,7 +586,7 @@ If you want a panel without title, set the title to C<undef>. Note that you can'
         </div>
     </div>
 
-The first shortcut, C<success>. This applies C<.panel-success>.
+Here, the C<success> strapping applies C<.panel-success> to the panel.
 
 
 
@@ -568,7 +604,7 @@ L<Bootstrap documentation|http://getbootstrap.com/css/#forms>
 
     # %arguments:
     cols => { $size => [ $label_columns, $input_columns ], ... },
-    @shortcuts
+    %strappings
     $fieldtype => $field_setting[],
     
     # $field_setting[]
@@ -580,7 +616,7 @@ L<Bootstrap documentation|http://getbootstrap.com/css/#forms>
     %html_attributes,
     %prepend,
     %append,
-    @shortcuts
+    %strappings
 
 B<C<$labeltext>>
 
@@ -601,9 +637,9 @@ that size, and C<$input_columns> is the number of columns used for the input fie
 
 You can defined the widths for one or more or all of the sizes.
 
-B<C<@shortcuts>>
+B<C<%strappings>>
 
-Optional. One or more shortcuts that you want applied to the C<.form-group> element.
+Optional hash. One or more strappings you want applied to the C<.form-group> element.
 
 B<C<$fieldtype>>
 
@@ -641,9 +677,9 @@ B<C<%prepend>> and B<C<%append>>
 
 Optional. Can be used individually or together. They are used to create L<input groups|http://getbootstrap.com/components/#input-groups>.
 
-B<C<@shortcuts>>
+B<C<%strappings>>
 
-Optional. All shortcuts you want applied to the C<input>.
+Optional. All strappings you want applied to the C<input>.
 
 =back
 
@@ -651,9 +687,10 @@ Optional. All shortcuts you want applied to the C<input>.
 
 =back
 
+
 =head3 Examples
 
-=head4 Basic form group
+B<Basic form group>
     
     %= formgroup 'Text test 1', text_field => ['test_text']
 
@@ -664,7 +701,8 @@ Optional. All shortcuts you want applied to the C<input>.
 
 The first item in the array ref is used for both C<id> and C<name>. Except...
 
-=head4 Input group (before), and large input field
+
+B<Input group (before), and large input field>
 
     %= formgroup 'Text test 4', text_field => ['test-text', append => '.00', large]
 
@@ -676,11 +714,12 @@ The first item in the array ref is used for both C<id> and C<name>. Except...
         </div>
     </div>
 
-Shortcuts can also be used in this context. Here C<large> applies C<.input-lg>.
+Strappings can also be used in this context. Here C<large> applies C<.input-lg>.
 
 If the input name (the first item in the text_field array ref) contains dashes, those are replaced (in the C<name>) to underscores.
 
-=head4 Input group (before and after), and with value
+
+B<Input group (before and after), and with value>
 
     %= formgroup 'Text test 5', text_field => ['test_text', '200', prepend => '$', append => '.00']
 
@@ -693,9 +732,10 @@ If the input name (the first item in the text_field array ref) contains dashes,
         </div>
     </div>
 
-The (optional) second item in the array ref is the value, if any, that should populate the input tag.
+Here, the second item in the C<text_field> array reference is a value that populates the C<input>.
 
-=head4 Large input group
+
+B<Large input group>
 
     %= formgroup 'Text test 6', text_field => ['test_text'], large
 
@@ -704,9 +744,10 @@ The (optional) second item in the array ref is the value, if any, that should po
         <input class="form-control" id="test_text" name="test_text" type="text" />
     </div>
 
-Note the difference with the earlier example. Here C<large> is outside the C<text_field> array ref, and therefore C<.form-group-lg> is applied to the form group. 
+Note the difference with the earlier example. Here C<large> is outside the C<text_field> array reference, and therefore C<.form-group-lg> is applied to the form group. 
 
-=head4 Horizontal form groups
+
+B<Horizontal form groups>
     
     %= formgroup 'Text test 8', text_field => ['test_text'], cols => { medium => [2, 10], small => [4, 8] }
 
@@ -719,7 +760,7 @@ Note the difference with the earlier example. Here C<large> is outside the C<tex
 
 If the C<form> is C<.form-horizontal>, you can set the column widths with the C<cols> attribute. The first item in each array ref is for the label, and the second for the input.
 
-(Note that in this context, C<medium> and C<large> are not shortcuts. Shortcuts don't take arguments.)
+(Note that in this context, C<medium> and C<large> are not short form strappings. Those don't take arguments.)
 
 
 
@@ -727,23 +768,91 @@ If the C<form> is C<.form-horizontal>, you can set the column widths with the C<
 
 L<Bootstrap documentation|http://getbootstrap.com/css/#buttons>
 
+=head3 Syntax
+
+    %= button $button_text, $url[], %arguments
+
+    # %arguments
+    %html_attributes,
+    %strappings
+
+B<C<$button_text>>
+
+Mandatory. The text on the button.
+
+B<C<$url[]>>
+
+Optional array reference. It is handed off to L<url_for|Mojolicious::Controller#url_for>, so with it this is
+basically L<link_to|Mojolicious::Plugin::TagHelpers#link_to> with Bootstrap classes.
+
+B<C<%arguments>>
+
+Optional hash.
+
+=over 4
+
+B<C<%html_attributes>>
+
+Optional hash of any html attributes you want to set on the button/link.
+
+B<C<%strappings>>
+
+Optional hash. Any strappings you want to set on the button/link.
+
+=back
+
+
+=head3 Examples
+
     %= button 'The example 5' => large, warning
 
     <button class="btn btn-lg btn-warning">The example 5</button>
 
-An ordinary button, with applied shortcuts.
+An ordinary button, with applied strappings.
     
     %= button 'The example 1' => ['http://www.example.com/'], small
 
     <a class="btn btn-sm" href="http://www.example.com/">The example 1</a>
 
-If the first argument after the button text is an array ref, it is used to populate C<href> and turns the button into a link. 
-The url is handed off L<url_for|Mojolicious::Controller#url_for>, so this is basically L<link_to|Mojolicious::Plugin::TagHelpers#link_to> with Bootstrap classes.
+With a url the button turns into a link.
 
 
 
 =head2 Tables
 
+=head3 Syntax
+
+    %= table $title, %arguments, begin
+           $body
+    %  end
+
+    # %arguments
+    %strappings
+    panel => $strappings{}
+
+B<C<$title>>
+
+Optional. If set the table will be wrapped in a panel. The table replaces the body.
+
+B<C<%arguments>>
+
+Optional hash:
+
+=over 4
+
+B<C<%strappings>>
+
+Optional. A hash of the strappings to apply to the table.
+
+B<C<panel =E<gt> $strappings{}>>
+
+An optional key-value pair. $strappings{} is hash reference containing any strapping you want to set on the panel.
+
+=back
+
+
+=head3 Examples
+
 L<Bootstrap documentation|http://getbootstrap.com/css/#tables>
 
     <%= table begin %>
@@ -766,14 +875,30 @@ A basic table.
 
 Several classes applied to the table.
 
+    %= table 'Heading Table 4', panel => { success }, condensed, begin
+        <tr><td>Table 4</td></tr>
+    %  end
+
+    <div class="panel panel-success">
+        <div class="panel-heading">
+            <h3 class="panel-title">Heading Table 4</h3>
+        </div>
+        <table class="table table-condensed">
+            <tr><td>Table 4</td></tr>
+        </table>
+    </div>
+
+A C<condensed> table wrapped in a C<success> panel.
+
+
 =head1 OPTIONS
 
 Some options are available:
 
     $app->plugin('BootstrapHelpers', {
         tag_prefix => 'bs',
-        shortcut_prefix => 'set',
-        init_shortcuts => 1,
+        short_strappings_prefix => 'set',
+        init_short_strappings => 1,
     });
 
 =head2 tag_prefix
@@ -797,22 +922,22 @@ If the option is set to any other string, the prefix is that string followed by
 =back
 
 
-=head2 shortcut_prefix
+=head2 short_strappings_prefix
 
 Default: C<undef>
 
-This is similar to C<tag_prefix>, but is instead applied to the shortcuts. The same rules applies.
+This is similar to C<tag_prefix>, but is instead applied to the short form strappings. The same rules applies.
 
 
-=head2 init_shortcuts
+=head2 init_short_strappings
 
 Default: C<1>
 
-If you don't want the shortcuts setup at all, set this option to a defined but false value.
+If you don't want the short form of strappings setup at all, set this option to a defined but false value.
 
-All functionality is available, but instead of C<warning> you must now use C<__warning =E<gt> 1>. That is why they are called shortcuts.
+All functionality is available, but instead of C<warning> you must now use C<<__warning => 1>>.
 
-With shortcuts turned off, sizes are only supported in longform: C<__xsmall>, C<__small>, C<__medium> and C<__large>.
+With short form turned off, sizes are still only supported in long form: C<__xsmall>, C<__small>, C<__medium> and C<__large>. The Bootstrap abbreviations (C<xs> - C<lg>) are not used.
 
 =head1 AUTHOR
 
